// Copyright 2018 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

part of flutter_mapbox_plugin;

typedef void MapCreatedCallback(MapboxMapController controller);

class MapboxMap extends StatefulWidget {
  const MapboxMap({
    @required
    this.initialCameraPosition,
    this.onMapCreated,
    this.cameraTargetBounds = CameraTargetBounds.unbounded,
    this.styleString,
    this.myLocationEnabled = false,
    this.myLocationTrackingMode = MyLocationTrackingMode.Tracking,
    this.onMapClick,
    this.symbolShowIndex = true,
  }) : assert(initialCameraPosition != null);

  final MapCreatedCallback onMapCreated;

  /// 地图视角初始化
  final CameraPosition initialCameraPosition;

  /// 地图视角边界
  final CameraTargetBounds cameraTargetBounds;

  /// 地图style
  final String styleString;


  ///地图是否开启定位
  final bool myLocationEnabled;

  /// 地图定位模式
  final MyLocationTrackingMode myLocationTrackingMode;

  ///大头针是否显示索引数字
  final bool symbolShowIndex;

  ///地图点击
  final OnMapClickCallback onMapClick;

  @override
  State createState() => _MapboxMapState();
}

class _MapboxMapState extends State<MapboxMap> {
  final Completer<MapboxMapController> _controller =
      Completer<MapboxMapController>();

  //  _MapboxMapOptions _mapboxMapOptions;

  @override
  Widget build(BuildContext context) {
    final Map<String, dynamic> creationParams = <String, dynamic>{
      'initialCameraPosition': widget.initialCameraPosition?._toMap(),
      'options': _MapboxMapOptions.fromWidget(widget).toMap(),
    };
    if (defaultTargetPlatform == TargetPlatform.android) {
      return AndroidView(
        viewType: 'plugins.flutter.io/mapbox_gl',
        onPlatformViewCreated: onPlatformViewCreated,
        creationParams: creationParams,
        creationParamsCodec: const StandardMessageCodec(),
      );
    } else if (defaultTargetPlatform == TargetPlatform.iOS) {
      return UiKitView(
        viewType: 'plugins.flutter.io/mapbox_gl',
        onPlatformViewCreated: onPlatformViewCreated,
        creationParams: creationParams,
        creationParamsCodec: const StandardMessageCodec(),
      );
    }

    return Text(
        '$defaultTargetPlatform is not yet supported by the maps plugin');
  }

  @override
  void initState() {
    super.initState();
   // _mapboxMapOptions = _MapboxMapOptions.fromWidget(widget);
  }

  Future<void> onPlatformViewCreated(int id) async {
    final MapboxMapController controller = await MapboxMapController.init(
        id, widget.initialCameraPosition,
        onMapClick: widget.onMapClick,);
    _controller.complete(controller);
    if (widget.onMapCreated != null) {
      widget.onMapCreated(controller);
    }
  }
}

/// Configuration options for the MapboxMaps user interface.
///
/// When used to change configuration, null values will be interpreted as
/// "do not change this configuration option".
class _MapboxMapOptions {
  _MapboxMapOptions({
    this.compassEnabled,
    this.cameraTargetBounds,
    this.styleString,
    this.myLocationEnabled,
    this.myLocationTrackingMode,
    this.symbolShowIndex,
  });

  static _MapboxMapOptions fromWidget(MapboxMap map) {
    return _MapboxMapOptions(
      cameraTargetBounds: map.cameraTargetBounds,
      styleString: map.styleString,
      myLocationEnabled: map.myLocationEnabled,
      myLocationTrackingMode: map.myLocationTrackingMode,
      symbolShowIndex: map.symbolShowIndex,
    );
  }

  final bool compassEnabled;

  final CameraTargetBounds cameraTargetBounds;

  final String styleString;

  final bool myLocationEnabled;

  final MyLocationTrackingMode myLocationTrackingMode;
  ///大头针是否显示索引数字
  final bool symbolShowIndex;
  Map<String, dynamic> toMap() {
    final Map<String, dynamic> optionsMap = <String, dynamic>{};

    void addIfNonNull(String fieldName, dynamic value) {
      if (value != null) {
        optionsMap[fieldName] = value;
      }
    }

    List<dynamic> pointToArray(Point fieldName) {
      if (fieldName != null) {
        return <dynamic>[fieldName.x, fieldName.y];
      }

      return null;
    }

    addIfNonNull('compassEnabled', compassEnabled);
    addIfNonNull('cameraTargetBounds', cameraTargetBounds?._toJson());
    addIfNonNull('styleString', styleString);
    addIfNonNull('myLocationEnabled', myLocationEnabled);
    addIfNonNull('myLocationTrackingMode', myLocationTrackingMode?.index);
    addIfNonNull('symbolShowIndex', symbolShowIndex);
    return optionsMap;
  }

  Map<String, dynamic> updatesMap(_MapboxMapOptions newOptions) {
    final Map<String, dynamic> prevOptionsMap = toMap();
    return newOptions.toMap()..removeWhere((String key, dynamic value) => prevOptionsMap[key] == value);
  }
}
